-- The ex7 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_ex7 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(6 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO256 : in std_logic_vector(7 downto 0);
  CO256_dly2 : in std_logic_vector(7 downto 0);
  X_RD_DATA : in std_logic_vector(2047 downto 0); 
  ReluAll : out std_logic_vector(1023 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_ex7;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_ex7 is

component dotpro_ex7
   Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(7 downto 0);
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO256is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(16 downto 0) );
end component;

component ROM_EW7
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(7 downto 0);
          DATA : Out std_logic_vector(383 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_000 : std_logic_vector(16 downto 0);

signal truncate_000 : std_logic_vector(9 downto 0);

signal Relu_000 : std_logic_vector(7 downto 0);

signal PixRomData : std_logic_vector(7 downto 0);
signal EW5Addr, EW5Addr_prev : std_logic_vector(7 downto 0);
signal EW5Sel : std_logic_vector(6 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(6 downto 0);

signal reg_000_000, reg_000_001, reg_000_002, reg_000_003, reg_000_004, reg_000_005, reg_000_006, reg_000_007, reg_000_008, reg_000_009 : std_logic_vector(7 downto 0);
signal reg_000_010, reg_000_011, reg_000_012, reg_000_013, reg_000_014, reg_000_015, reg_000_016, reg_000_017, reg_000_018, reg_000_019 : std_logic_vector(7 downto 0);
signal reg_000_020, reg_000_021, reg_000_022, reg_000_023, reg_000_024, reg_000_025, reg_000_026, reg_000_027, reg_000_028, reg_000_029 : std_logic_vector(7 downto 0);
signal reg_000_030, reg_000_031, reg_000_032, reg_000_033, reg_000_034, reg_000_035, reg_000_036, reg_000_037, reg_000_038, reg_000_039 : std_logic_vector(7 downto 0);
signal reg_000_040, reg_000_041, reg_000_042, reg_000_043, reg_000_044, reg_000_045, reg_000_046, reg_000_047, reg_000_048, reg_000_049 : std_logic_vector(7 downto 0);
signal reg_000_050, reg_000_051, reg_000_052, reg_000_053, reg_000_054, reg_000_055, reg_000_056, reg_000_057, reg_000_058, reg_000_059 : std_logic_vector(7 downto 0);
signal reg_000_060, reg_000_061, reg_000_062, reg_000_063, reg_000_064, reg_000_065, reg_000_066, reg_000_067, reg_000_068, reg_000_069 : std_logic_vector(7 downto 0);
signal reg_000_070, reg_000_071, reg_000_072, reg_000_073, reg_000_074, reg_000_075, reg_000_076, reg_000_077, reg_000_078, reg_000_079 : std_logic_vector(7 downto 0);
signal reg_000_080, reg_000_081, reg_000_082, reg_000_083, reg_000_084, reg_000_085, reg_000_086, reg_000_087, reg_000_088, reg_000_089 : std_logic_vector(7 downto 0);
signal reg_000_090, reg_000_091, reg_000_092, reg_000_093, reg_000_094, reg_000_095, reg_000_096, reg_000_097, reg_000_098, reg_000_099 : std_logic_vector(7 downto 0);
signal reg_000_100, reg_000_101, reg_000_102, reg_000_103, reg_000_104, reg_000_105, reg_000_106, reg_000_107, reg_000_108, reg_000_109 : std_logic_vector(7 downto 0);
signal reg_000_110, reg_000_111, reg_000_112, reg_000_113, reg_000_114, reg_000_115, reg_000_116, reg_000_117, reg_000_118, reg_000_119 : std_logic_vector(7 downto 0);
signal reg_000_120, reg_000_121, reg_000_122, reg_000_123, reg_000_124, reg_000_125, reg_000_126, reg_000_127 : std_logic_vector(7 downto 0);

signal enb_000, enb_001, enb_002, enb_003, enb_004, enb_005, enb_006, enb_007, enb_008, enb_009 : std_logic;
signal enb_010, enb_011, enb_012, enb_013, enb_014, enb_015, enb_016, enb_017, enb_018, enb_019 : std_logic;
signal enb_020, enb_021, enb_022, enb_023, enb_024, enb_025, enb_026, enb_027, enb_028, enb_029 : std_logic;
signal enb_030, enb_031, enb_032, enb_033, enb_034, enb_035, enb_036, enb_037, enb_038, enb_039 : std_logic;
signal enb_040, enb_041, enb_042, enb_043, enb_044, enb_045, enb_046, enb_047, enb_048, enb_049 : std_logic;
signal enb_050, enb_051, enb_052, enb_053, enb_054, enb_055, enb_056, enb_057, enb_058, enb_059 : std_logic;
signal enb_060, enb_061, enb_062, enb_063, enb_064, enb_065, enb_066, enb_067, enb_068, enb_069 : std_logic;
signal enb_070, enb_071, enb_072, enb_073, enb_074, enb_075, enb_076, enb_077, enb_078, enb_079 : std_logic;
signal enb_080, enb_081, enb_082, enb_083, enb_084, enb_085, enb_086, enb_087, enb_088, enb_089 : std_logic;
signal enb_090, enb_091, enb_092, enb_093, enb_094, enb_095, enb_096, enb_097, enb_098, enb_099 : std_logic;
signal enb_100, enb_101, enb_102, enb_103, enb_104, enb_105, enb_106, enb_107, enb_108, enb_109 : std_logic;
signal enb_110, enb_111, enb_112, enb_113, enb_114, enb_115, enb_116, enb_117, enb_118, enb_119 : std_logic;
signal enb_120, enb_121, enb_122, enb_123, enb_124, enb_125, enb_126, enb_127 : std_logic;
signal CO9is1, CO256is0 : std_logic;
signal EW5Data : std_logic_vector(383 downto 0);
signal coeff_000: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
CO256is0 <= '1' when CO256 = 1 else '0'; -- for lay15 one more dly

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

EW5Sel <= OutChSel_dly2;

---------------------------------------------------------
  PixRomData <= 
X_RD_DATA( 2047 downto 2040 ) when CO256_dly2 = "00000000" else 
X_RD_DATA( 2039 downto 2032 ) when CO256_dly2 = "00000001" else 
X_RD_DATA( 2031 downto 2024 ) when CO256_dly2 = "00000010" else 
X_RD_DATA( 2023 downto 2016 ) when CO256_dly2 = "00000011" else 
X_RD_DATA( 2015 downto 2008 ) when CO256_dly2 = "00000100" else 
X_RD_DATA( 2007 downto 2000 ) when CO256_dly2 = "00000101" else 
X_RD_DATA( 1999 downto 1992 ) when CO256_dly2 = "00000110" else 
X_RD_DATA( 1991 downto 1984 ) when CO256_dly2 = "00000111" else 
X_RD_DATA( 1983 downto 1976 ) when CO256_dly2 = "00001000" else 
X_RD_DATA( 1975 downto 1968 ) when CO256_dly2 = "00001001" else 
X_RD_DATA( 1967 downto 1960 ) when CO256_dly2 = "00001010" else 
X_RD_DATA( 1959 downto 1952 ) when CO256_dly2 = "00001011" else 
X_RD_DATA( 1951 downto 1944 ) when CO256_dly2 = "00001100" else 
X_RD_DATA( 1943 downto 1936 ) when CO256_dly2 = "00001101" else 
X_RD_DATA( 1935 downto 1928 ) when CO256_dly2 = "00001110" else 
X_RD_DATA( 1927 downto 1920 ) when CO256_dly2 = "00001111" else 
X_RD_DATA( 1919 downto 1912 ) when CO256_dly2 = "00010000" else 
X_RD_DATA( 1911 downto 1904 ) when CO256_dly2 = "00010001" else 
X_RD_DATA( 1903 downto 1896 ) when CO256_dly2 = "00010010" else 
X_RD_DATA( 1895 downto 1888 ) when CO256_dly2 = "00010011" else 
X_RD_DATA( 1887 downto 1880 ) when CO256_dly2 = "00010100" else 
X_RD_DATA( 1879 downto 1872 ) when CO256_dly2 = "00010101" else 
X_RD_DATA( 1871 downto 1864 ) when CO256_dly2 = "00010110" else 
X_RD_DATA( 1863 downto 1856 ) when CO256_dly2 = "00010111" else 
X_RD_DATA( 1855 downto 1848 ) when CO256_dly2 = "00011000" else 
X_RD_DATA( 1847 downto 1840 ) when CO256_dly2 = "00011001" else 
X_RD_DATA( 1839 downto 1832 ) when CO256_dly2 = "00011010" else 
X_RD_DATA( 1831 downto 1824 ) when CO256_dly2 = "00011011" else 
X_RD_DATA( 1823 downto 1816 ) when CO256_dly2 = "00011100" else 
X_RD_DATA( 1815 downto 1808 ) when CO256_dly2 = "00011101" else 
X_RD_DATA( 1807 downto 1800 ) when CO256_dly2 = "00011110" else 
X_RD_DATA( 1799 downto 1792 ) when CO256_dly2 = "00011111" else 
X_RD_DATA( 1791 downto 1784 ) when CO256_dly2 = "00100000" else 
X_RD_DATA( 1783 downto 1776 ) when CO256_dly2 = "00100001" else 
X_RD_DATA( 1775 downto 1768 ) when CO256_dly2 = "00100010" else 
X_RD_DATA( 1767 downto 1760 ) when CO256_dly2 = "00100011" else 
X_RD_DATA( 1759 downto 1752 ) when CO256_dly2 = "00100100" else 
X_RD_DATA( 1751 downto 1744 ) when CO256_dly2 = "00100101" else 
X_RD_DATA( 1743 downto 1736 ) when CO256_dly2 = "00100110" else 
X_RD_DATA( 1735 downto 1728 ) when CO256_dly2 = "00100111" else 
X_RD_DATA( 1727 downto 1720 ) when CO256_dly2 = "00101000" else 
X_RD_DATA( 1719 downto 1712 ) when CO256_dly2 = "00101001" else 
X_RD_DATA( 1711 downto 1704 ) when CO256_dly2 = "00101010" else 
X_RD_DATA( 1703 downto 1696 ) when CO256_dly2 = "00101011" else 
X_RD_DATA( 1695 downto 1688 ) when CO256_dly2 = "00101100" else 
X_RD_DATA( 1687 downto 1680 ) when CO256_dly2 = "00101101" else 
X_RD_DATA( 1679 downto 1672 ) when CO256_dly2 = "00101110" else 
X_RD_DATA( 1671 downto 1664 ) when CO256_dly2 = "00101111" else 
X_RD_DATA( 1663 downto 1656 ) when CO256_dly2 = "00110000" else 
X_RD_DATA( 1655 downto 1648 ) when CO256_dly2 = "00110001" else 
X_RD_DATA( 1647 downto 1640 ) when CO256_dly2 = "00110010" else 
X_RD_DATA( 1639 downto 1632 ) when CO256_dly2 = "00110011" else 
X_RD_DATA( 1631 downto 1624 ) when CO256_dly2 = "00110100" else 
X_RD_DATA( 1623 downto 1616 ) when CO256_dly2 = "00110101" else 
X_RD_DATA( 1615 downto 1608 ) when CO256_dly2 = "00110110" else 
X_RD_DATA( 1607 downto 1600 ) when CO256_dly2 = "00110111" else 
X_RD_DATA( 1599 downto 1592 ) when CO256_dly2 = "00111000" else 
X_RD_DATA( 1591 downto 1584 ) when CO256_dly2 = "00111001" else 
X_RD_DATA( 1583 downto 1576 ) when CO256_dly2 = "00111010" else 
X_RD_DATA( 1575 downto 1568 ) when CO256_dly2 = "00111011" else 
X_RD_DATA( 1567 downto 1560 ) when CO256_dly2 = "00111100" else 
X_RD_DATA( 1559 downto 1552 ) when CO256_dly2 = "00111101" else 
X_RD_DATA( 1551 downto 1544 ) when CO256_dly2 = "00111110" else 
X_RD_DATA( 1543 downto 1536 ) when CO256_dly2 = "00111111" else 
X_RD_DATA( 1535 downto 1528 ) when CO256_dly2 = "01000000" else 
X_RD_DATA( 1527 downto 1520 ) when CO256_dly2 = "01000001" else 
X_RD_DATA( 1519 downto 1512 ) when CO256_dly2 = "01000010" else 
X_RD_DATA( 1511 downto 1504 ) when CO256_dly2 = "01000011" else 
X_RD_DATA( 1503 downto 1496 ) when CO256_dly2 = "01000100" else 
X_RD_DATA( 1495 downto 1488 ) when CO256_dly2 = "01000101" else 
X_RD_DATA( 1487 downto 1480 ) when CO256_dly2 = "01000110" else 
X_RD_DATA( 1479 downto 1472 ) when CO256_dly2 = "01000111" else 
X_RD_DATA( 1471 downto 1464 ) when CO256_dly2 = "01001000" else 
X_RD_DATA( 1463 downto 1456 ) when CO256_dly2 = "01001001" else 
X_RD_DATA( 1455 downto 1448 ) when CO256_dly2 = "01001010" else 
X_RD_DATA( 1447 downto 1440 ) when CO256_dly2 = "01001011" else 
X_RD_DATA( 1439 downto 1432 ) when CO256_dly2 = "01001100" else 
X_RD_DATA( 1431 downto 1424 ) when CO256_dly2 = "01001101" else 
X_RD_DATA( 1423 downto 1416 ) when CO256_dly2 = "01001110" else 
X_RD_DATA( 1415 downto 1408 ) when CO256_dly2 = "01001111" else 
X_RD_DATA( 1407 downto 1400 ) when CO256_dly2 = "01010000" else 
X_RD_DATA( 1399 downto 1392 ) when CO256_dly2 = "01010001" else 
X_RD_DATA( 1391 downto 1384 ) when CO256_dly2 = "01010010" else 
X_RD_DATA( 1383 downto 1376 ) when CO256_dly2 = "01010011" else 
X_RD_DATA( 1375 downto 1368 ) when CO256_dly2 = "01010100" else 
X_RD_DATA( 1367 downto 1360 ) when CO256_dly2 = "01010101" else 
X_RD_DATA( 1359 downto 1352 ) when CO256_dly2 = "01010110" else 
X_RD_DATA( 1351 downto 1344 ) when CO256_dly2 = "01010111" else 
X_RD_DATA( 1343 downto 1336 ) when CO256_dly2 = "01011000" else 
X_RD_DATA( 1335 downto 1328 ) when CO256_dly2 = "01011001" else 
X_RD_DATA( 1327 downto 1320 ) when CO256_dly2 = "01011010" else 
X_RD_DATA( 1319 downto 1312 ) when CO256_dly2 = "01011011" else 
X_RD_DATA( 1311 downto 1304 ) when CO256_dly2 = "01011100" else 
X_RD_DATA( 1303 downto 1296 ) when CO256_dly2 = "01011101" else 
X_RD_DATA( 1295 downto 1288 ) when CO256_dly2 = "01011110" else 
X_RD_DATA( 1287 downto 1280 ) when CO256_dly2 = "01011111" else 
X_RD_DATA( 1279 downto 1272 ) when CO256_dly2 = "01100000" else 
X_RD_DATA( 1271 downto 1264 ) when CO256_dly2 = "01100001" else 
X_RD_DATA( 1263 downto 1256 ) when CO256_dly2 = "01100010" else 
X_RD_DATA( 1255 downto 1248 ) when CO256_dly2 = "01100011" else 
X_RD_DATA( 1247 downto 1240 ) when CO256_dly2 = "01100100" else 
X_RD_DATA( 1239 downto 1232 ) when CO256_dly2 = "01100101" else 
X_RD_DATA( 1231 downto 1224 ) when CO256_dly2 = "01100110" else 
X_RD_DATA( 1223 downto 1216 ) when CO256_dly2 = "01100111" else 
X_RD_DATA( 1215 downto 1208 ) when CO256_dly2 = "01101000" else 
X_RD_DATA( 1207 downto 1200 ) when CO256_dly2 = "01101001" else 
X_RD_DATA( 1199 downto 1192 ) when CO256_dly2 = "01101010" else 
X_RD_DATA( 1191 downto 1184 ) when CO256_dly2 = "01101011" else 
X_RD_DATA( 1183 downto 1176 ) when CO256_dly2 = "01101100" else 
X_RD_DATA( 1175 downto 1168 ) when CO256_dly2 = "01101101" else 
X_RD_DATA( 1167 downto 1160 ) when CO256_dly2 = "01101110" else 
X_RD_DATA( 1159 downto 1152 ) when CO256_dly2 = "01101111" else 
X_RD_DATA( 1151 downto 1144 ) when CO256_dly2 = "01110000" else 
X_RD_DATA( 1143 downto 1136 ) when CO256_dly2 = "01110001" else 
X_RD_DATA( 1135 downto 1128 ) when CO256_dly2 = "01110010" else 
X_RD_DATA( 1127 downto 1120 ) when CO256_dly2 = "01110011" else 
X_RD_DATA( 1119 downto 1112 ) when CO256_dly2 = "01110100" else 
X_RD_DATA( 1111 downto 1104 ) when CO256_dly2 = "01110101" else 
X_RD_DATA( 1103 downto 1096 ) when CO256_dly2 = "01110110" else 
X_RD_DATA( 1095 downto 1088 ) when CO256_dly2 = "01110111" else 
X_RD_DATA( 1087 downto 1080 ) when CO256_dly2 = "01111000" else 
X_RD_DATA( 1079 downto 1072 ) when CO256_dly2 = "01111001" else 
X_RD_DATA( 1071 downto 1064 ) when CO256_dly2 = "01111010" else 
X_RD_DATA( 1063 downto 1056 ) when CO256_dly2 = "01111011" else 
X_RD_DATA( 1055 downto 1048 ) when CO256_dly2 = "01111100" else 
X_RD_DATA( 1047 downto 1040 ) when CO256_dly2 = "01111101" else 
X_RD_DATA( 1039 downto 1032 ) when CO256_dly2 = "01111110" else 
X_RD_DATA( 1031 downto 1024 ) when CO256_dly2 = "01111111" else 
X_RD_DATA( 1023 downto 1016 ) when CO256_dly2 = "10000000" else 
X_RD_DATA( 1015 downto 1008 ) when CO256_dly2 = "10000001" else 
X_RD_DATA( 1007 downto 1000 ) when CO256_dly2 = "10000010" else 
X_RD_DATA( 999 downto 992 ) when CO256_dly2 = "10000011" else 
X_RD_DATA( 991 downto 984 ) when CO256_dly2 = "10000100" else 
X_RD_DATA( 983 downto 976 ) when CO256_dly2 = "10000101" else 
X_RD_DATA( 975 downto 968 ) when CO256_dly2 = "10000110" else 
X_RD_DATA( 967 downto 960 ) when CO256_dly2 = "10000111" else 
X_RD_DATA( 959 downto 952 ) when CO256_dly2 = "10001000" else 
X_RD_DATA( 951 downto 944 ) when CO256_dly2 = "10001001" else 
X_RD_DATA( 943 downto 936 ) when CO256_dly2 = "10001010" else 
X_RD_DATA( 935 downto 928 ) when CO256_dly2 = "10001011" else 
X_RD_DATA( 927 downto 920 ) when CO256_dly2 = "10001100" else 
X_RD_DATA( 919 downto 912 ) when CO256_dly2 = "10001101" else 
X_RD_DATA( 911 downto 904 ) when CO256_dly2 = "10001110" else 
X_RD_DATA( 903 downto 896 ) when CO256_dly2 = "10001111" else 
X_RD_DATA( 895 downto 888 ) when CO256_dly2 = "10010000" else 
X_RD_DATA( 887 downto 880 ) when CO256_dly2 = "10010001" else 
X_RD_DATA( 879 downto 872 ) when CO256_dly2 = "10010010" else 
X_RD_DATA( 871 downto 864 ) when CO256_dly2 = "10010011" else 
X_RD_DATA( 863 downto 856 ) when CO256_dly2 = "10010100" else 
X_RD_DATA( 855 downto 848 ) when CO256_dly2 = "10010101" else 
X_RD_DATA( 847 downto 840 ) when CO256_dly2 = "10010110" else 
X_RD_DATA( 839 downto 832 ) when CO256_dly2 = "10010111" else 
X_RD_DATA( 831 downto 824 ) when CO256_dly2 = "10011000" else 
X_RD_DATA( 823 downto 816 ) when CO256_dly2 = "10011001" else 
X_RD_DATA( 815 downto 808 ) when CO256_dly2 = "10011010" else 
X_RD_DATA( 807 downto 800 ) when CO256_dly2 = "10011011" else 
X_RD_DATA( 799 downto 792 ) when CO256_dly2 = "10011100" else 
X_RD_DATA( 791 downto 784 ) when CO256_dly2 = "10011101" else 
X_RD_DATA( 783 downto 776 ) when CO256_dly2 = "10011110" else 
X_RD_DATA( 775 downto 768 ) when CO256_dly2 = "10011111" else 
X_RD_DATA( 767 downto 760 ) when CO256_dly2 = "10100000" else 
X_RD_DATA( 759 downto 752 ) when CO256_dly2 = "10100001" else 
X_RD_DATA( 751 downto 744 ) when CO256_dly2 = "10100010" else 
X_RD_DATA( 743 downto 736 ) when CO256_dly2 = "10100011" else 
X_RD_DATA( 735 downto 728 ) when CO256_dly2 = "10100100" else 
X_RD_DATA( 727 downto 720 ) when CO256_dly2 = "10100101" else 
X_RD_DATA( 719 downto 712 ) when CO256_dly2 = "10100110" else 
X_RD_DATA( 711 downto 704 ) when CO256_dly2 = "10100111" else 
X_RD_DATA( 703 downto 696 ) when CO256_dly2 = "10101000" else 
X_RD_DATA( 695 downto 688 ) when CO256_dly2 = "10101001" else 
X_RD_DATA( 687 downto 680 ) when CO256_dly2 = "10101010" else 
X_RD_DATA( 679 downto 672 ) when CO256_dly2 = "10101011" else 
X_RD_DATA( 671 downto 664 ) when CO256_dly2 = "10101100" else 
X_RD_DATA( 663 downto 656 ) when CO256_dly2 = "10101101" else 
X_RD_DATA( 655 downto 648 ) when CO256_dly2 = "10101110" else 
X_RD_DATA( 647 downto 640 ) when CO256_dly2 = "10101111" else 
X_RD_DATA( 639 downto 632 ) when CO256_dly2 = "10110000" else 
X_RD_DATA( 631 downto 624 ) when CO256_dly2 = "10110001" else 
X_RD_DATA( 623 downto 616 ) when CO256_dly2 = "10110010" else 
X_RD_DATA( 615 downto 608 ) when CO256_dly2 = "10110011" else 
X_RD_DATA( 607 downto 600 ) when CO256_dly2 = "10110100" else 
X_RD_DATA( 599 downto 592 ) when CO256_dly2 = "10110101" else 
X_RD_DATA( 591 downto 584 ) when CO256_dly2 = "10110110" else 
X_RD_DATA( 583 downto 576 ) when CO256_dly2 = "10110111" else 
X_RD_DATA( 575 downto 568 ) when CO256_dly2 = "10111000" else 
X_RD_DATA( 567 downto 560 ) when CO256_dly2 = "10111001" else 
X_RD_DATA( 559 downto 552 ) when CO256_dly2 = "10111010" else 
X_RD_DATA( 551 downto 544 ) when CO256_dly2 = "10111011" else 
X_RD_DATA( 543 downto 536 ) when CO256_dly2 = "10111100" else 
X_RD_DATA( 535 downto 528 ) when CO256_dly2 = "10111101" else 
X_RD_DATA( 527 downto 520 ) when CO256_dly2 = "10111110" else 
X_RD_DATA( 519 downto 512 ) when CO256_dly2 = "10111111" else 
X_RD_DATA( 511 downto 504 ) when CO256_dly2 = "11000000" else 
X_RD_DATA( 503 downto 496 ) when CO256_dly2 = "11000001" else 
X_RD_DATA( 495 downto 488 ) when CO256_dly2 = "11000010" else 
X_RD_DATA( 487 downto 480 ) when CO256_dly2 = "11000011" else 
X_RD_DATA( 479 downto 472 ) when CO256_dly2 = "11000100" else 
X_RD_DATA( 471 downto 464 ) when CO256_dly2 = "11000101" else 
X_RD_DATA( 463 downto 456 ) when CO256_dly2 = "11000110" else 
X_RD_DATA( 455 downto 448 ) when CO256_dly2 = "11000111" else 
X_RD_DATA( 447 downto 440 ) when CO256_dly2 = "11001000" else 
X_RD_DATA( 439 downto 432 ) when CO256_dly2 = "11001001" else 
X_RD_DATA( 431 downto 424 ) when CO256_dly2 = "11001010" else 
X_RD_DATA( 423 downto 416 ) when CO256_dly2 = "11001011" else 
X_RD_DATA( 415 downto 408 ) when CO256_dly2 = "11001100" else 
X_RD_DATA( 407 downto 400 ) when CO256_dly2 = "11001101" else 
X_RD_DATA( 399 downto 392 ) when CO256_dly2 = "11001110" else 
X_RD_DATA( 391 downto 384 ) when CO256_dly2 = "11001111" else 
X_RD_DATA( 383 downto 376 ) when CO256_dly2 = "11010000" else 
X_RD_DATA( 375 downto 368 ) when CO256_dly2 = "11010001" else 
X_RD_DATA( 367 downto 360 ) when CO256_dly2 = "11010010" else 
X_RD_DATA( 359 downto 352 ) when CO256_dly2 = "11010011" else 
X_RD_DATA( 351 downto 344 ) when CO256_dly2 = "11010100" else 
X_RD_DATA( 343 downto 336 ) when CO256_dly2 = "11010101" else 
X_RD_DATA( 335 downto 328 ) when CO256_dly2 = "11010110" else 
X_RD_DATA( 327 downto 320 ) when CO256_dly2 = "11010111" else 
X_RD_DATA( 319 downto 312 ) when CO256_dly2 = "11011000" else 
X_RD_DATA( 311 downto 304 ) when CO256_dly2 = "11011001" else 
X_RD_DATA( 303 downto 296 ) when CO256_dly2 = "11011010" else 
X_RD_DATA( 295 downto 288 ) when CO256_dly2 = "11011011" else 
X_RD_DATA( 287 downto 280 ) when CO256_dly2 = "11011100" else 
X_RD_DATA( 279 downto 272 ) when CO256_dly2 = "11011101" else 
X_RD_DATA( 271 downto 264 ) when CO256_dly2 = "11011110" else 
X_RD_DATA( 263 downto 256 ) when CO256_dly2 = "11011111" else 
X_RD_DATA( 255 downto 248 ) when CO256_dly2 = "11100000" else 
X_RD_DATA( 247 downto 240 ) when CO256_dly2 = "11100001" else 
X_RD_DATA( 239 downto 232 ) when CO256_dly2 = "11100010" else 
X_RD_DATA( 231 downto 224 ) when CO256_dly2 = "11100011" else 
X_RD_DATA( 223 downto 216 ) when CO256_dly2 = "11100100" else 
X_RD_DATA( 215 downto 208 ) when CO256_dly2 = "11100101" else 
X_RD_DATA( 207 downto 200 ) when CO256_dly2 = "11100110" else 
X_RD_DATA( 199 downto 192 ) when CO256_dly2 = "11100111" else 
X_RD_DATA( 191 downto 184 ) when CO256_dly2 = "11101000" else 
X_RD_DATA( 183 downto 176 ) when CO256_dly2 = "11101001" else 
X_RD_DATA( 175 downto 168 ) when CO256_dly2 = "11101010" else 
X_RD_DATA( 167 downto 160 ) when CO256_dly2 = "11101011" else 
X_RD_DATA( 159 downto 152 ) when CO256_dly2 = "11101100" else 
X_RD_DATA( 151 downto 144 ) when CO256_dly2 = "11101101" else 
X_RD_DATA( 143 downto 136 ) when CO256_dly2 = "11101110" else 
X_RD_DATA( 135 downto 128 ) when CO256_dly2 = "11101111" else 
X_RD_DATA( 127 downto 120 ) when CO256_dly2 = "11110000" else 
X_RD_DATA( 119 downto 112 ) when CO256_dly2 = "11110001" else 
X_RD_DATA( 111 downto 104 ) when CO256_dly2 = "11110010" else 
X_RD_DATA( 103 downto 96 ) when CO256_dly2 = "11110011" else 
X_RD_DATA( 95 downto 88 ) when CO256_dly2 = "11110100" else 
X_RD_DATA( 87 downto 80 ) when CO256_dly2 = "11110101" else 
X_RD_DATA( 79 downto 72 ) when CO256_dly2 = "11110110" else 
X_RD_DATA( 71 downto 64 ) when CO256_dly2 = "11110111" else 
X_RD_DATA( 63 downto 56 ) when CO256_dly2 = "11111000" else 
X_RD_DATA( 55 downto 48 ) when CO256_dly2 = "11111001" else 
X_RD_DATA( 47 downto 40 ) when CO256_dly2 = "11111010" else 
X_RD_DATA( 39 downto 32 ) when CO256_dly2 = "11111011" else 
X_RD_DATA( 31 downto 24 ) when CO256_dly2 = "11111100" else 
X_RD_DATA( 23 downto 16 ) when CO256_dly2 = "111111101" else 
X_RD_DATA( 15 downto 8 ) when CO256_dly2 = "111111110" else 
X_RD_DATA( 7 downto 0 ); -- when CO256_dly2 = "11111111" else 


EW5Addr_prev <= CO256; -- CO256 x 1

---------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                EW5Addr <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                EW5Addr <= EW5Addr_prev;
        end if; 
end process;


---------------------------------------------------------
 -- removed CO9=8 for lay15
enb_000 <= '1' when EW5Sel = 1 and CO256 = 255 else '0';
enb_001 <= '1' when EW5Sel = 2 and CO256 = 255 else '0';
enb_002 <= '1' when EW5Sel = 3 and CO256 = 255 else '0';
enb_003 <= '1' when EW5Sel = 4 and CO256 = 255 else '0';
enb_004 <= '1' when EW5Sel = 5 and CO256 = 255 else '0';
enb_005 <= '1' when EW5Sel = 6 and CO256 = 255 else '0';
enb_006 <= '1' when EW5Sel = 7 and CO256 = 255 else '0';
enb_007 <= '1' when EW5Sel = 8 and CO256 = 255 else '0';
enb_008 <= '1' when EW5Sel = 9 and CO256 = 255 else '0';
enb_009 <= '1' when EW5Sel = 10 and CO256 = 255 else '0';
enb_010 <= '1' when EW5Sel = 11 and CO256 = 255 else '0';
enb_011 <= '1' when EW5Sel = 12 and CO256 = 255 else '0';
enb_012 <= '1' when EW5Sel = 13 and CO256 = 255 else '0';
enb_013 <= '1' when EW5Sel = 14 and CO256 = 255 else '0';
enb_014 <= '1' when EW5Sel = 15 and CO256 = 255 else '0';
enb_015 <= '1' when EW5Sel = 16 and CO256 = 255 else '0';
enb_016 <= '1' when EW5Sel = 17 and CO256 = 255 else '0';
enb_017 <= '1' when EW5Sel = 18 and CO256 = 255 else '0';
enb_018 <= '1' when EW5Sel = 19 and CO256 = 255 else '0';
enb_019 <= '1' when EW5Sel = 20 and CO256 = 255 else '0';
enb_020 <= '1' when EW5Sel = 21 and CO256 = 255 else '0';
enb_021 <= '1' when EW5Sel = 22 and CO256 = 255 else '0';
enb_022 <= '1' when EW5Sel = 23 and CO256 = 255 else '0';
enb_023 <= '1' when EW5Sel = 24 and CO256 = 255 else '0';
enb_024 <= '1' when EW5Sel = 25 and CO256 = 255 else '0';
enb_025 <= '1' when EW5Sel = 26 and CO256 = 255 else '0';
enb_026 <= '1' when EW5Sel = 27 and CO256 = 255 else '0';
enb_027 <= '1' when EW5Sel = 28 and CO256 = 255 else '0';
enb_028 <= '1' when EW5Sel = 29 and CO256 = 255 else '0';
enb_029 <= '1' when EW5Sel = 30 and CO256 = 255 else '0';
enb_030 <= '1' when EW5Sel = 31 and CO256 = 255 else '0';
enb_031 <= '1' when EW5Sel = 32 and CO256 = 255 else '0';
enb_032 <= '1' when EW5Sel = 33 and CO256 = 255 else '0';
enb_033 <= '1' when EW5Sel = 34 and CO256 = 255 else '0';
enb_034 <= '1' when EW5Sel = 35 and CO256 = 255 else '0';
enb_035 <= '1' when EW5Sel = 36 and CO256 = 255 else '0';
enb_036 <= '1' when EW5Sel = 37 and CO256 = 255 else '0';
enb_037 <= '1' when EW5Sel = 38 and CO256 = 255 else '0';
enb_038 <= '1' when EW5Sel = 39 and CO256 = 255 else '0';
enb_039 <= '1' when EW5Sel = 40 and CO256 = 255 else '0';
enb_040 <= '1' when EW5Sel = 41 and CO256 = 255 else '0';
enb_041 <= '1' when EW5Sel = 42 and CO256 = 255 else '0';
enb_042 <= '1' when EW5Sel = 43 and CO256 = 255 else '0';
enb_043 <= '1' when EW5Sel = 44 and CO256 = 255 else '0';
enb_044 <= '1' when EW5Sel = 45 and CO256 = 255 else '0';
enb_045 <= '1' when EW5Sel = 46 and CO256 = 255 else '0';
enb_046 <= '1' when EW5Sel = 47 and CO256 = 255 else '0';
enb_047 <= '1' when EW5Sel = 48 and CO256 = 255 else '0';
enb_048 <= '1' when EW5Sel = 49 and CO256 = 255 else '0';
enb_049 <= '1' when EW5Sel = 50 and CO256 = 255 else '0';
enb_050 <= '1' when EW5Sel = 51 and CO256 = 255 else '0';
enb_051 <= '1' when EW5Sel = 52 and CO256 = 255 else '0';
enb_052 <= '1' when EW5Sel = 53 and CO256 = 255 else '0';
enb_053 <= '1' when EW5Sel = 54 and CO256 = 255 else '0';
enb_054 <= '1' when EW5Sel = 55 and CO256 = 255 else '0';
enb_055 <= '1' when EW5Sel = 56 and CO256 = 255 else '0';
enb_056 <= '1' when EW5Sel = 57 and CO256 = 255 else '0';
enb_057 <= '1' when EW5Sel = 58 and CO256 = 255 else '0';
enb_058 <= '1' when EW5Sel = 59 and CO256 = 255 else '0';
enb_059 <= '1' when EW5Sel = 60 and CO256 = 255 else '0';
enb_060 <= '1' when EW5Sel = 61 and CO256 = 255 else '0';
enb_061 <= '1' when EW5Sel = 62 and CO256 = 255 else '0';
enb_062 <= '1' when EW5Sel = 63 and CO256 = 255 else '0';
enb_063 <= '1' when EW5Sel = 64 and CO256 = 255 else '0';
enb_064 <= '1' when EW5Sel = 65 and CO256 = 255 else '0';
enb_065 <= '1' when EW5Sel = 66 and CO256 = 255 else '0';
enb_066 <= '1' when EW5Sel = 67 and CO256 = 255 else '0';
enb_067 <= '1' when EW5Sel = 68 and CO256 = 255 else '0';
enb_068 <= '1' when EW5Sel = 69 and CO256 = 255 else '0';
enb_069 <= '1' when EW5Sel = 70 and CO256 = 255 else '0';
enb_070 <= '1' when EW5Sel = 71 and CO256 = 255 else '0';
enb_071 <= '1' when EW5Sel = 72 and CO256 = 255 else '0';
enb_072 <= '1' when EW5Sel = 73 and CO256 = 255 else '0';
enb_073 <= '1' when EW5Sel = 74 and CO256 = 255 else '0';
enb_074 <= '1' when EW5Sel = 75 and CO256 = 255 else '0';
enb_075 <= '1' when EW5Sel = 76 and CO256 = 255 else '0';
enb_076 <= '1' when EW5Sel = 77 and CO256 = 255 else '0';
enb_077 <= '1' when EW5Sel = 78 and CO256 = 255 else '0';
enb_078 <= '1' when EW5Sel = 79 and CO256 = 255 else '0';
enb_079 <= '1' when EW5Sel = 80 and CO256 = 255 else '0';
enb_080 <= '1' when EW5Sel = 81 and CO256 = 255 else '0';
enb_081 <= '1' when EW5Sel = 82 and CO256 = 255 else '0';
enb_082 <= '1' when EW5Sel = 83 and CO256 = 255 else '0';
enb_083 <= '1' when EW5Sel = 84 and CO256 = 255 else '0';
enb_084 <= '1' when EW5Sel = 85 and CO256 = 255 else '0';
enb_085 <= '1' when EW5Sel = 86 and CO256 = 255 else '0';
enb_086 <= '1' when EW5Sel = 87 and CO256 = 255 else '0';
enb_087 <= '1' when EW5Sel = 88 and CO256 = 255 else '0';
enb_088 <= '1' when EW5Sel = 89 and CO256 = 255 else '0';
enb_089 <= '1' when EW5Sel = 90 and CO256 = 255 else '0';
enb_090 <= '1' when EW5Sel = 91 and CO256 = 255 else '0';
enb_091 <= '1' when EW5Sel = 92 and CO256 = 255 else '0';
enb_092 <= '1' when EW5Sel = 93 and CO256 = 255 else '0';
enb_093 <= '1' when EW5Sel = 94 and CO256 = 255 else '0';
enb_094 <= '1' when EW5Sel = 95 and CO256 = 255 else '0';
enb_095 <= '1' when EW5Sel = 96 and CO256 = 255 else '0';
enb_096 <= '1' when EW5Sel = 97 and CO256 = 255 else '0';
enb_097 <= '1' when EW5Sel = 98 and CO256 = 255 else '0';
enb_098 <= '1' when EW5Sel = 99 and CO256 = 255 else '0';
enb_099 <= '1' when EW5Sel = 100 and CO256 = 255 else '0';
enb_100 <= '1' when EW5Sel = 101 and CO256 = 255 else '0';
enb_101 <= '1' when EW5Sel = 102 and CO256 = 255 else '0';
enb_102 <= '1' when EW5Sel = 103 and CO256 = 255 else '0';
enb_103 <= '1' when EW5Sel = 104 and CO256 = 255 else '0';
enb_104 <= '1' when EW5Sel = 105 and CO256 = 255 else '0';
enb_105 <= '1' when EW5Sel = 106 and CO256 = 255 else '0';
enb_106 <= '1' when EW5Sel = 107 and CO256 = 255 else '0';
enb_107 <= '1' when EW5Sel = 108 and CO256 = 255 else '0';
enb_108 <= '1' when EW5Sel = 109 and CO256 = 255 else '0';
enb_109 <= '1' when EW5Sel = 110 and CO256 = 255 else '0';
enb_110 <= '1' when EW5Sel = 111 and CO256 = 255 else '0';
enb_111 <= '1' when EW5Sel = 112 and CO256 = 255 else '0';
enb_112 <= '1' when EW5Sel = 113 and CO256 = 255 else '0';
enb_113 <= '1' when EW5Sel = 114 and CO256 = 255 else '0';
enb_114 <= '1' when EW5Sel = 115 and CO256 = 255 else '0';
enb_115 <= '1' when EW5Sel = 116 and CO256 = 255 else '0';
enb_116 <= '1' when EW5Sel = 117 and CO256 = 255 else '0';
enb_117 <= '1' when EW5Sel = 118 and CO256 = 255 else '0';
enb_118 <= '1' when EW5Sel = 119 and CO256 = 255 else '0';
enb_119 <= '1' when EW5Sel = 120 and CO256 = 255 else '0';
enb_120 <= '1' when EW5Sel = 121 and CO256 = 255 else '0';
enb_121 <= '1' when EW5Sel = 122 and CO256 = 255 else '0';
enb_122 <= '1' when EW5Sel = 123 and CO256 = 255 else '0';
enb_123 <= '1' when EW5Sel = 124 and CO256 = 255 else '0';
enb_124 <= '1' when EW5Sel = 125 and CO256 = 255 else '0';
enb_125 <= '1' when EW5Sel = 126 and CO256 = 255 else '0';
enb_126 <= '1' when EW5Sel = 127 and CO256 = 255 else '0';
enb_127 <= '1' when EW5Sel = 0 and CO256 = 255 else '0';

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_000_000 <= Relu_000;
    elsif(enb_001 = '1') then reg_000_001 <= Relu_000;
    elsif(enb_002 = '1') then reg_000_002 <= Relu_000;
    elsif(enb_003 = '1') then reg_000_003 <= Relu_000;
    elsif(enb_004 = '1') then reg_000_004 <= Relu_000;
    elsif(enb_005 = '1') then reg_000_005 <= Relu_000;
    elsif(enb_006 = '1') then reg_000_006 <= Relu_000;
    elsif(enb_007 = '1') then reg_000_007 <= Relu_000;
    elsif(enb_008 = '1') then reg_000_008 <= Relu_000;
    elsif(enb_009 = '1') then reg_000_009 <= Relu_000;
    elsif(enb_010 = '1') then reg_000_010 <= Relu_000;
    elsif(enb_011 = '1') then reg_000_011 <= Relu_000;
    elsif(enb_012 = '1') then reg_000_012 <= Relu_000;
    elsif(enb_013 = '1') then reg_000_013 <= Relu_000;
    elsif(enb_014 = '1') then reg_000_014 <= Relu_000;
    elsif(enb_015 = '1') then reg_000_015 <= Relu_000;
    elsif(enb_016 = '1') then reg_000_016 <= Relu_000;
    elsif(enb_017 = '1') then reg_000_017 <= Relu_000;
    elsif(enb_018 = '1') then reg_000_018 <= Relu_000;
    elsif(enb_019 = '1') then reg_000_019 <= Relu_000;
    elsif(enb_020 = '1') then reg_000_020 <= Relu_000;
    elsif(enb_021 = '1') then reg_000_021 <= Relu_000;
    elsif(enb_022 = '1') then reg_000_022 <= Relu_000;
    elsif(enb_023 = '1') then reg_000_023 <= Relu_000;
    elsif(enb_024 = '1') then reg_000_024 <= Relu_000;
    elsif(enb_025 = '1') then reg_000_025 <= Relu_000;
    elsif(enb_026 = '1') then reg_000_026 <= Relu_000;
    elsif(enb_027 = '1') then reg_000_027 <= Relu_000;
    elsif(enb_028 = '1') then reg_000_028 <= Relu_000;
    elsif(enb_029 = '1') then reg_000_029 <= Relu_000;
    elsif(enb_030 = '1') then reg_000_030 <= Relu_000;
    elsif(enb_031 = '1') then reg_000_031 <= Relu_000;
    elsif(enb_032 = '1') then reg_000_032 <= Relu_000;
    elsif(enb_033 = '1') then reg_000_033 <= Relu_000;
    elsif(enb_034 = '1') then reg_000_034 <= Relu_000;
    elsif(enb_035 = '1') then reg_000_035 <= Relu_000;
    elsif(enb_036 = '1') then reg_000_036 <= Relu_000;
    elsif(enb_037 = '1') then reg_000_037 <= Relu_000;
    elsif(enb_038 = '1') then reg_000_038 <= Relu_000;
    elsif(enb_039 = '1') then reg_000_039 <= Relu_000;
    elsif(enb_040 = '1') then reg_000_040 <= Relu_000;
    elsif(enb_041 = '1') then reg_000_041 <= Relu_000;
    elsif(enb_042 = '1') then reg_000_042 <= Relu_000;
    elsif(enb_043 = '1') then reg_000_043 <= Relu_000;
    elsif(enb_044 = '1') then reg_000_044 <= Relu_000;
    elsif(enb_045 = '1') then reg_000_045 <= Relu_000;
    elsif(enb_046 = '1') then reg_000_046 <= Relu_000;
    elsif(enb_047 = '1') then reg_000_047 <= Relu_000;
    elsif(enb_048 = '1') then reg_000_048 <= Relu_000;
    elsif(enb_049 = '1') then reg_000_049 <= Relu_000;
    elsif(enb_050 = '1') then reg_000_050 <= Relu_000;
    elsif(enb_051 = '1') then reg_000_051 <= Relu_000;
    elsif(enb_052 = '1') then reg_000_052 <= Relu_000;
    elsif(enb_053 = '1') then reg_000_053 <= Relu_000;
    elsif(enb_054 = '1') then reg_000_054 <= Relu_000;
    elsif(enb_055 = '1') then reg_000_055 <= Relu_000;
    elsif(enb_056 = '1') then reg_000_056 <= Relu_000;
    elsif(enb_057 = '1') then reg_000_057 <= Relu_000;
    elsif(enb_058 = '1') then reg_000_058 <= Relu_000;
    elsif(enb_059 = '1') then reg_000_059 <= Relu_000;
    elsif(enb_060 = '1') then reg_000_060 <= Relu_000;
    elsif(enb_061 = '1') then reg_000_061 <= Relu_000;
    elsif(enb_062 = '1') then reg_000_062 <= Relu_000;
    elsif(enb_063 = '1') then reg_000_063 <= Relu_000;
    elsif(enb_064 = '1') then reg_000_064 <= Relu_000;
    elsif(enb_065 = '1') then reg_000_065 <= Relu_000;
    elsif(enb_066 = '1') then reg_000_066 <= Relu_000;
    elsif(enb_067 = '1') then reg_000_067 <= Relu_000;
    elsif(enb_068 = '1') then reg_000_068 <= Relu_000;
    elsif(enb_069 = '1') then reg_000_069 <= Relu_000;
    elsif(enb_070 = '1') then reg_000_070 <= Relu_000;
    elsif(enb_071 = '1') then reg_000_071 <= Relu_000;
    elsif(enb_072 = '1') then reg_000_072 <= Relu_000;
    elsif(enb_073 = '1') then reg_000_073 <= Relu_000;
    elsif(enb_074 = '1') then reg_000_074 <= Relu_000;
    elsif(enb_075 = '1') then reg_000_075 <= Relu_000;
    elsif(enb_076 = '1') then reg_000_076 <= Relu_000;
    elsif(enb_077 = '1') then reg_000_077 <= Relu_000;
    elsif(enb_078 = '1') then reg_000_078 <= Relu_000;
    elsif(enb_079 = '1') then reg_000_079 <= Relu_000;
    elsif(enb_080 = '1') then reg_000_080 <= Relu_000;
    elsif(enb_081 = '1') then reg_000_081 <= Relu_000;
    elsif(enb_082 = '1') then reg_000_082 <= Relu_000;
    elsif(enb_083 = '1') then reg_000_083 <= Relu_000;
    elsif(enb_084 = '1') then reg_000_084 <= Relu_000;
    elsif(enb_085 = '1') then reg_000_085 <= Relu_000;
    elsif(enb_086 = '1') then reg_000_086 <= Relu_000;
    elsif(enb_087 = '1') then reg_000_087 <= Relu_000;
    elsif(enb_088 = '1') then reg_000_088 <= Relu_000;
    elsif(enb_089 = '1') then reg_000_089 <= Relu_000;
    elsif(enb_090 = '1') then reg_000_090 <= Relu_000;
    elsif(enb_091 = '1') then reg_000_091 <= Relu_000;
    elsif(enb_092 = '1') then reg_000_092 <= Relu_000;
    elsif(enb_093 = '1') then reg_000_093 <= Relu_000;
    elsif(enb_094 = '1') then reg_000_094 <= Relu_000;
    elsif(enb_095 = '1') then reg_000_095 <= Relu_000;
    elsif(enb_096 = '1') then reg_000_096 <= Relu_000;
    elsif(enb_097 = '1') then reg_000_097 <= Relu_000;
    elsif(enb_098 = '1') then reg_000_098 <= Relu_000;
    elsif(enb_099 = '1') then reg_000_099 <= Relu_000;
    elsif(enb_100 = '1') then reg_000_100 <= Relu_000;
    elsif(enb_101 = '1') then reg_000_101 <= Relu_000;
    elsif(enb_102 = '1') then reg_000_102 <= Relu_000;
    elsif(enb_103 = '1') then reg_000_103 <= Relu_000;
    elsif(enb_104 = '1') then reg_000_104 <= Relu_000;
    elsif(enb_105 = '1') then reg_000_105 <= Relu_000;
    elsif(enb_106 = '1') then reg_000_106 <= Relu_000;
    elsif(enb_107 = '1') then reg_000_107 <= Relu_000;
    elsif(enb_108 = '1') then reg_000_108 <= Relu_000;
    elsif(enb_109 = '1') then reg_000_109 <= Relu_000;
    elsif(enb_110 = '1') then reg_000_110 <= Relu_000;
    elsif(enb_111 = '1') then reg_000_111 <= Relu_000;
    elsif(enb_112 = '1') then reg_000_112 <= Relu_000;
    elsif(enb_113 = '1') then reg_000_113 <= Relu_000;
    elsif(enb_114 = '1') then reg_000_114 <= Relu_000;
    elsif(enb_115 = '1') then reg_000_115 <= Relu_000;
    elsif(enb_116 = '1') then reg_000_116 <= Relu_000;
    elsif(enb_117 = '1') then reg_000_117 <= Relu_000;
    elsif(enb_118 = '1') then reg_000_118 <= Relu_000;
    elsif(enb_119 = '1') then reg_000_119 <= Relu_000;
    elsif(enb_120 = '1') then reg_000_120 <= Relu_000;
    elsif(enb_121 = '1') then reg_000_121 <= Relu_000;
    elsif(enb_122 = '1') then reg_000_122 <= Relu_000;
    elsif(enb_123 = '1') then reg_000_123 <= Relu_000;
    elsif(enb_124 = '1') then reg_000_124 <= Relu_000;
    elsif(enb_125 = '1') then reg_000_125 <= Relu_000;
    elsif(enb_126 = '1') then reg_000_126 <= Relu_000;
    elsif(enb_127 = '1') then reg_000_127 <= Relu_000;
    end if;
  end if;  
end process;


-------------------------------------------------------
ROM_EW7_inst : ROM_EW7 port map (
  CLK => CLK,
  ADDR => EW5Addr,
  DATA => EW5Data );

---------------------------------------------------------

coeff_000 <= EW5Data( 383 downto 381 ) when EW5Sel = "0000000" else EW5Data( 380 downto 378 ) when EW5Sel = "0000001" else EW5Data( 377 downto 375 ) when EW5Sel = "0000010" else EW5Data( 374 downto 372 ) when EW5Sel = "0000011" else EW5Data( 371 downto 369 ) when EW5Sel = "0000100" else EW5Data( 368 downto 366 ) when EW5Sel = "0000101" else EW5Data( 365 downto 363 ) when EW5Sel = "0000110" else EW5Data( 362 downto 360 ) when EW5Sel = "0000111" else EW5Data( 359 downto 357 ) when EW5Sel = "0001000" else EW5Data( 356 downto 354 ) when EW5Sel = "0001001" else EW5Data( 353 downto 351 ) when EW5Sel = "0001010" else EW5Data( 350 downto 348 ) when EW5Sel = "0001011" else EW5Data( 347 downto 345 ) when EW5Sel = "0001100" else EW5Data( 344 downto 342 ) when EW5Sel = "0001101" else EW5Data( 341 downto 339 ) when EW5Sel = "0001110" else EW5Data( 338 downto 336 ) when EW5Sel = "0001111" 

else EW5Data( 335 downto 333 ) when EW5Sel = "0010000" else EW5Data( 332 downto 330 ) when EW5Sel = "0010001" else EW5Data( 329 downto 327 ) when EW5Sel = "0010010" else EW5Data( 326 downto 324 ) when EW5Sel = "0010011" else EW5Data( 323 downto 321 ) when EW5Sel = "0010100" else EW5Data( 320 downto 318 ) when EW5Sel = "0010101" else EW5Data( 317 downto 315 ) when EW5Sel = "0010110" else EW5Data( 314 downto 312 ) when EW5Sel = "0010111" else EW5Data( 311 downto 309 ) when EW5Sel = "0011000" else EW5Data( 308 downto 306 ) when EW5Sel = "0011001" else EW5Data( 305 downto 303 ) when EW5Sel = "0011010" else EW5Data( 302 downto 300 ) when EW5Sel = "0011011" else EW5Data( 299 downto 297 ) when EW5Sel = "0011100" else EW5Data( 296 downto 294 ) when EW5Sel = "0011101" else EW5Data( 293 downto 291 ) when EW5Sel = "0011110" else EW5Data( 290 downto 288 ) when EW5Sel = "0011111" 

else EW5Data( 287 downto 285 ) when EW5Sel = "0100000" else EW5Data( 284 downto 282 ) when EW5Sel = "0100001" else EW5Data( 281 downto 279 ) when EW5Sel = "0100010" else EW5Data( 278 downto 276 ) when EW5Sel = "0100011" else EW5Data( 275 downto 273 ) when EW5Sel = "0100100" else EW5Data( 272 downto 270 ) when EW5Sel = "0100101" else EW5Data( 269 downto 267 ) when EW5Sel = "0100110" else EW5Data( 266 downto 264 ) when EW5Sel = "0100111" else EW5Data( 263 downto 261 ) when EW5Sel = "0101000" else EW5Data( 260 downto 258 ) when EW5Sel = "0101001" else EW5Data( 257 downto 255 ) when EW5Sel = "0101010" else EW5Data( 254 downto 252 ) when EW5Sel = "0101011" else EW5Data( 251 downto 249 ) when EW5Sel = "0101100" else EW5Data( 248 downto 246 ) when EW5Sel = "0101101" else EW5Data( 245 downto 243 ) when EW5Sel = "0101110" else EW5Data( 242 downto 240 ) when EW5Sel = "0101111" 

else EW5Data( 239 downto 237 ) when EW5Sel = "0110000" else EW5Data( 236 downto 234 ) when EW5Sel = "0110001" else EW5Data( 233 downto 231 ) when EW5Sel = "0110010" else EW5Data( 230 downto 228 ) when EW5Sel = "0110011" else EW5Data( 227 downto 225 ) when EW5Sel = "0110100" else EW5Data( 224 downto 222 ) when EW5Sel = "0110101" else EW5Data( 221 downto 219 ) when EW5Sel = "0110110" else EW5Data( 218 downto 216 ) when EW5Sel = "0110111" else EW5Data( 215 downto 213 ) when EW5Sel = "0111000" else EW5Data( 212 downto 210 ) when EW5Sel = "0111001" else EW5Data( 209 downto 207 ) when EW5Sel = "0111010" else EW5Data( 206 downto 204 ) when EW5Sel = "0111011" else EW5Data( 203 downto 201 ) when EW5Sel = "0111100" else EW5Data( 200 downto 198 ) when EW5Sel = "0111101" else EW5Data( 197 downto 195 ) when EW5Sel = "0111110" else EW5Data( 194 downto 192 ) when EW5Sel = "0111111" 

else EW5Data( 191 downto 189 ) when EW5Sel = "1000000" else EW5Data( 188 downto 186 ) when EW5Sel = "1000001" else EW5Data( 185 downto 183 ) when EW5Sel = "1000010" else EW5Data( 182 downto 180 ) when EW5Sel = "1000011" else EW5Data( 179 downto 177 ) when EW5Sel = "1000100" else EW5Data( 176 downto 174 ) when EW5Sel = "1000101" else EW5Data( 173 downto 171 ) when EW5Sel = "1000110" else EW5Data( 170 downto 168 ) when EW5Sel = "1000111" else EW5Data( 167 downto 165 ) when EW5Sel = "1001000" else EW5Data( 164 downto 162 ) when EW5Sel = "1001001" else EW5Data( 161 downto 159 ) when EW5Sel = "1001010" else EW5Data( 158 downto 156 ) when EW5Sel = "1001011" else EW5Data( 155 downto 153 ) when EW5Sel = "1001100" else EW5Data( 152 downto 150 ) when EW5Sel = "1001101" else EW5Data( 149 downto 147 ) when EW5Sel = "1001110" else EW5Data( 146 downto 144 ) when EW5Sel = "1001111" 

else EW5Data( 143 downto 141 ) when EW5Sel = "1010000" else EW5Data( 140 downto 138 ) when EW5Sel = "1010001" else EW5Data( 137 downto 135 ) when EW5Sel = "1010010" else EW5Data( 134 downto 132 ) when EW5Sel = "1010011" else EW5Data( 131 downto 129 ) when EW5Sel = "1010100" else EW5Data( 128 downto 126 ) when EW5Sel = "1010101" else EW5Data( 125 downto 123 ) when EW5Sel = "1010110" else EW5Data( 122 downto 120 ) when EW5Sel = "1010111" else EW5Data( 119 downto 117 ) when EW5Sel = "1011000" else EW5Data( 116 downto 114 ) when EW5Sel = "1011001" else EW5Data( 113 downto 111 ) when EW5Sel = "1011010" else EW5Data( 110 downto 108 ) when EW5Sel = "1011011" else EW5Data( 107 downto 105 ) when EW5Sel = "1011100" else EW5Data( 104 downto 102 ) when EW5Sel = "1011101" else EW5Data( 101 downto 99 ) when EW5Sel = "1011110" else EW5Data( 98 downto 96 ) when EW5Sel = "1011111" 

else EW5Data( 95 downto 93 ) when EW5Sel = "1100000" else EW5Data( 92 downto 90 ) when EW5Sel = "1100001" else EW5Data( 89 downto 87 ) when EW5Sel = "1100010" else EW5Data( 86 downto 84 ) when EW5Sel = "1100011" else EW5Data( 83 downto 81 ) when EW5Sel = "1100100" else EW5Data( 80 downto 78 ) when EW5Sel = "1100101" else EW5Data( 77 downto 75 ) when EW5Sel = "1100110" else EW5Data( 74 downto 72 ) when EW5Sel = "1100111" else EW5Data( 71 downto 69 ) when EW5Sel = "1101000" else EW5Data( 68 downto 66 ) when EW5Sel = "1101001" else EW5Data( 65 downto 63 ) when EW5Sel = "1101010" else EW5Data( 62 downto 60 ) when EW5Sel = "1101011" else EW5Data( 59 downto 57 ) when EW5Sel = "1101100" else EW5Data( 56 downto 54 ) when EW5Sel = "1101101" else EW5Data( 53 downto 51 ) when EW5Sel = "1101110" else EW5Data( 50 downto 48 ) when EW5Sel = "1101111" 

else EW5Data( 47 downto 45 ) when EW5Sel = "1110000" else EW5Data( 44 downto 42 ) when EW5Sel = "1110001" else EW5Data( 41 downto 39 ) when EW5Sel = "1110010" else EW5Data( 38 downto 36 ) when EW5Sel = "1110011" else EW5Data( 35 downto 33 ) when EW5Sel = "1110100" else EW5Data( 32 downto 30 ) when EW5Sel = "1110101" else EW5Data( 29 downto 27 ) when EW5Sel = "1110110" else EW5Data( 26 downto 24 ) when EW5Sel = "1110111" else EW5Data( 23 downto 21 ) when EW5Sel = "1111000" else EW5Data( 20 downto 18 ) when EW5Sel = "1111001" else EW5Data( 17 downto 15 ) when EW5Sel = "1111010" else EW5Data( 14 downto 12 ) when EW5Sel = "1111011" else EW5Data( 11 downto 9 ) when EW5Sel = "1111100" else EW5Data( 8 downto 6 ) when EW5Sel = "1111101" else EW5Data( 5 downto 3 ) when EW5Sel = "1111110" else EW5Data( 2 downto 0 );--- when EW5Sel = "1111111" 


---------------------------------------------------------
dotpro_ex7_000 : dotpro_ex7 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_000,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_000 );

-------------------------------------------------------
  truncate_000 <= mac_000(14 downto 5);

---------------------------------------------------------
  Relu_000 <= (others => '0') when truncate_000 < 0 else
             (others => '1') when truncate_000 > 255 else
             truncate_000(7 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_000_000 & reg_000_001 & reg_000_002 & reg_000_003 & reg_000_004 & reg_000_005 & reg_000_006 & reg_000_007 & reg_000_008 & reg_000_009 & 
	reg_000_010 & reg_000_011 & reg_000_012 & reg_000_013 & reg_000_014 & reg_000_015 & reg_000_016 & reg_000_017 & reg_000_018 & reg_000_019 & 
	reg_000_020 & reg_000_021 & reg_000_022 & reg_000_023 & reg_000_024 & reg_000_025 & reg_000_026 & reg_000_027 & reg_000_028 & reg_000_029 & 
	reg_000_030 & reg_000_031 & reg_000_032 & reg_000_033 & reg_000_034 & reg_000_035 & reg_000_036 & reg_000_037 & reg_000_038 & reg_000_039 & 
	reg_000_040 & reg_000_041 & reg_000_042 & reg_000_043 & reg_000_044 & reg_000_045 & reg_000_046 & reg_000_047 & reg_000_048 & reg_000_049 & 
	reg_000_050 & reg_000_051 & reg_000_052 & reg_000_053 & reg_000_054 & reg_000_055 & reg_000_056 & reg_000_057 & reg_000_058 & reg_000_059 & 
	reg_000_060 & reg_000_061 & reg_000_062 & reg_000_063 & reg_000_064 & reg_000_065 & reg_000_066 & reg_000_067 & reg_000_068 & reg_000_069 & 
	reg_000_070 & reg_000_071 & reg_000_072 & reg_000_073 & reg_000_074 & reg_000_075 & reg_000_076 & reg_000_077 & reg_000_078 & reg_000_079 & 
	reg_000_080 & reg_000_081 & reg_000_082 & reg_000_083 & reg_000_084 & reg_000_085 & reg_000_086 & reg_000_087 & reg_000_088 & reg_000_089 & 
	reg_000_090 & reg_000_091 & reg_000_092 & reg_000_093 & reg_000_094 & reg_000_095 & reg_000_096 & reg_000_097 & reg_000_098 & reg_000_099 & 
	reg_000_100 & reg_000_101 & reg_000_102 & reg_000_103 & reg_000_104 & reg_000_105 & reg_000_106 & reg_000_107 & reg_000_108 & reg_000_109 & 
	reg_000_110 & reg_000_111 & reg_000_112 & reg_000_113 & reg_000_114 & reg_000_115 & reg_000_116 & reg_000_117 & reg_000_118 & reg_000_119 & 
	reg_000_120 & reg_000_121 & reg_000_122 & reg_000_123 & reg_000_124 & reg_000_125 & reg_000_126 & reg_000_127 ;


ReluAllWrN <= not enb_000; -- ex5 is ok with this 000

end RTL;

